@@ -22,7 +22,7 @@ Copyright 2003-2007 Peter Beerli, Tallahassee FL
  works and are not responsible for any damage you may cause or have.
  
  
-$Id: data.c 1712 2010-08-20 04:17:16Z beerli $
+$Id: data.c 1742 2010-10-24 17:03:07Z beerli $
 
 -----------------------------------
 add this to the microsatellite data, this will allow to use
@@ -80,7 +80,7 @@ void read_indname (FILE * file, data_fmt * data, long pop, long ind,
                    long locus, long nmlength);
 void read_popdata (FILE * file, data_fmt * data, long pop,
                    option_fmt * options);
-void read_microalleles (FILE * infile, data_fmt * data, long pop, long ind);
+void read_microalleles (FILE * infile, data_fmt * data, option_fmt *options, long pop, long ind);
 void read_alleles (FILE * infile, data_fmt * data, long pop, long ind);
 long read_ind_seq (FILE * infile, data_fmt * data, option_fmt * options,
                    long locus, long pop, long ind, long baseread);
@@ -467,7 +467,7 @@ init_data_structure1 (data_fmt ** data, option_fmt * options)
         (*data)->seq = (seqmodel_fmt **) mycalloc (1, sizeof (seqmodel_fmt *));
         (*data)->seq[0] = (seqmodel_fmt *) mycalloc (1, sizeof (seqmodel_fmt));
         (*data)->popnames =(char **) mymalloc (sizeof (char *) * numpop);
-        (*data)->popnames[0] =(char *) mycalloc (numpop * LINESIZE,sizeof(char));
+        (*data)->popnames[0] =(char *) mycalloc (numpop * STRSIZE,sizeof(char));
         (*data)->indnames = (char ****) mymalloc (sizeof (char ***) * numpop);
         (*data)->numind = (long **) mymalloc (sizeof (long *) * numpop);
         (*data)->numind[0] = (long *) mymalloc (sizeof (long) * numpop * loci);
@@ -477,7 +477,7 @@ init_data_structure1 (data_fmt ** data, option_fmt * options)
 	(*data)->locusname = (char **) mycalloc(loci, sizeof(char*));
         for (pop = 1; pop < numpop; pop++)
         {
-	  (*data)->popnames[pop] = (*data)->popnames[0] + pop * LINESIZE;
+	  (*data)->popnames[pop] = (*data)->popnames[0] + pop * STRSIZE;
 	  (*data)->numind[pop] = (*data)->numind[0] + pop * loci;
 	  (*data)->numalleles[pop] =  (*data)->numalleles[0] + pop * loci;
         }
@@ -749,13 +749,14 @@ read_sites (data_fmt * data, world_fmt *world, option_fmt *options)
 {
   char * val=NULL;
   boolean linked=TRUE;
+  long allocbufsize=LINESIZE;
     long locus;
     char *input;
     char * word;
     long z=0; //z is adding subloci and is reset with a closing ')'
     long zz=0; //zz is used for mutationmodels and is not reset at closing ')'
-    input = (char *) mycalloc(LONGLINESIZE ,sizeof(char));
-    word = (char *) mycalloc(LONGLINESIZE ,sizeof(char));
+    input = (char *) mycalloc(allocbufsize ,sizeof(char));
+    word = (char *) mycalloc(allocbufsize ,sizeof(char));
     // check whether the microsats are not repeats but fragmentlength  
     data->has_repeats = TRUE;
     for (locus = 0; locus < data->loci; locus++)
@@ -769,7 +770,7 @@ read_sites (data_fmt * data, world_fmt *world, option_fmt *options)
 	      // line is microsatellite instruction
 	      if(input[2]=='M')
 		{
-		  FGETS(input,LINESIZE,data->infile); // read the line 
+		  FGETS2(&input,&allocbufsize,data->infile); // read the line 
 		  locus--;
 		  long oldi=0;
 		  long i;
@@ -777,7 +778,6 @@ read_sites (data_fmt * data, world_fmt *world, option_fmt *options)
 		  data->has_repeats = FALSE;
 		  for(i = 0 ; i < data->loci; i++)
 		    {
-		      memset(word,'0', sizeof(char)*LINESIZE);
 		      len += read_word_delim(input+len,word," ;,\t");
 		      if(word[0]!='\0')
 			{
@@ -792,7 +792,7 @@ read_sites (data_fmt * data, world_fmt *world, option_fmt *options)
 		}
 	      else
 		{
-		  FGETS(input,LINESIZE,data->infile); // read the line 
+		  FGETS2(&input,&allocbufsize,data->infile); // read the line 
 		  locus--;
 		}
 	      myfree(input);
@@ -802,7 +802,7 @@ read_sites (data_fmt * data, world_fmt *world, option_fmt *options)
 	  else
 	    {
 	      // line is a comment
-	      FGETS(input,LINESIZE,data->infile); // read the line 
+	      FGETS2(&input,&allocbufsize,data->infile); // read the line 
 	      locus--;
 	      continue;
 	    }
@@ -914,25 +914,9 @@ read_sites (data_fmt * data, world_fmt *world, option_fmt *options)
 	  usererror ("will crash anyway, so I stop now\n");
         }
     }
-    FGETS(input,LINESIZE,data->infile);
+    FGETS2(&input,&allocbufsize,data->infile);
     while(input[0]=='#')
-      FGETS(input,LINESIZE,data->infile);
-    //    if(linked)
-    //  data->seq[0]->links[locus] = TRUE;
-    //else
-    //  data->seq[0]->links[locus] = FALSE;
-    //if(strchr("bmsnuh",input[0]))
-    //  {
-    //	val = input+1;
-    //  }
-    //else
-    //  {
-    //	val = input;
-    //	while(!strchr("123456789",*val))
-    //	      val++;
-    //      }
-    //data->seq[0]->sites[locus] = atoi (val);
-    
+      FGETS2(&input,&allocbufsize,data->infile);
     myfree(input);
     myfree(word);
 }
@@ -1171,6 +1155,7 @@ read_popheader (FILE * infile, data_fmt * data, option_fmt * options, long pop,
     
     translate (data->popnames[pop], ' ', '_');
     translate (data->popnames[pop], '\t', '_');
+    unpad(data->popnames[pop],"_");
     myfree(input);
 }
 
@@ -1219,7 +1204,7 @@ read_popdata (FILE * infile, data_fmt * data, long pop, option_fmt * options)
 		if (data->dlm == '\0')
 		  read_alleles (infile, data, pop, ind);
 		else
-		  read_microalleles (infile, data, pop, ind);
+		  read_microalleles (infile, data, options, pop, ind);
 		break;
 	      case 's':
 	      case 'n':
@@ -1273,6 +1258,7 @@ void len2repeat(char *a1, long rlen, long locus, data_fmt *data)
   long a=0;
   if((a=la1 % rlen) != 0)
     {
+      //      long correction = (((((float) rlen) / 2.) < a) ? (-a) : (rlen/2 == a ? (UNIF_RANDUM()<0.5 ? -a : a) : (rlen - a))); 
       long correction = (((((float) rlen) / 2.) < a) ? (-a) : (rlen/2 == a ? (UNIF_RANDUM()<0.5 ? -a : a) : (rlen - a))); 
       //long nla = check_list(la1,la1+correction, locus, data);
       long nla = (la1+correction)/rlen;
@@ -1286,7 +1272,7 @@ void len2repeat(char *a1, long rlen, long locus, data_fmt *data)
 
 
 void
-read_microalleles (FILE * infile, data_fmt * data, long pop, long ind)
+read_microalleles (FILE * infile, data_fmt * data, option_fmt *options, long pop, long ind)
 {
     char *input, *isave, dlm[2], ddlm[2], *p, *a, *a1, *a2;
     long locus, i;
@@ -1334,14 +1320,8 @@ read_microalleles (FILE * infile, data_fmt * data, long pop, long ind)
         {
             strcpy (a2, a1);
         }
-	if(data->repeatlength[locus]!=0)
-	  {
-	    long rlen = data->repeatlength[locus];
-	    len2repeat(a1, rlen, locus, data);
-	    len2repeat(a2, rlen, locus, data);
-          }
-        strcpy (data->yy[pop][ind][locus][0], a1);
-        strcpy (data->yy[pop][ind][locus][1], a2);
+        strncpy (data->yy[pop][ind][locus][0], a1, options->allelenmlength);
+        strncpy (data->yy[pop][ind][locus][1], a2, options->allelenmlength);
     }
     myfree(a);
     myfree(a1);
@@ -1759,7 +1739,7 @@ print_data_summary (FILE * file, world_fmt * world, option_fmt * options,
   total = (long *) mycalloc(data->loci,sizeof(long));
   totalmiss = (long *) mycalloc(data->loci,sizeof(long));
   fprintf (file, "\nSummary of data:\n");
-  fprintf(file, "Title: %s\n",options->title);
+  fprintf(file, "Title:%54.54s\n",options->title);
   
   switch (options->datatype)
     {
@@ -1785,20 +1765,21 @@ print_data_summary (FILE * file, world_fmt * world, option_fmt * options,
       strcpy (dstring, "Unknown data [ERROR]");
       break;
     }
-  fprintf (file, "Data file:             %s\n", options->infilename);
-  fprintf (file, "Datatype:              %20s\n", dstring);
+  fprintf (file, "Data file:  %48.48s\n", options->infilename);
+  fprintf (file, "Datatype:   %48.48s\n", dstring);
   if(!data->has_repeats)
     {
       fprintf (file, "[Fragment length is translated to repeats]\n");
     }
 
-  fprintf (file, "Number of loci:        %20li\n\n",
+  fprintf (file, "Number of loci:                         %20li\n\n",
 	   data->loci);
   if(options->has_datefile)
     {
-      fprintf (file, "Sample dates:          %s\n", options->datefilename);
-      fprintf (file, "Generations per year:  %f\n", options->generation_year);
-      fprintf (file, "Mutationrate per year: %.10g", options->mutationrate_year[0]);
+      fprintf (file, "Title:%54.54s\n",options->title);
+      fprintf (file, "Sample dates:%46.46s\n", options->datefilename);
+      fprintf (file, "Generations per year:                        %5.5f\n", options->generation_year);
+      fprintf (file, "Mutationrate per year:  %.10g", options->mutationrate_year[0]);
       for(locus=1; locus < options->mutationrate_year_numalloc; locus++)
 	{
 	  if(locus % 4 == 0)
@@ -1926,7 +1907,6 @@ void print_spectra(world_fmt * world, option_fmt * options,data_fmt * data)
   for (locus = 0; locus < data->loci; locus++)
     {
       maxalleles[locus] = findAllele(data,"\0",locus);
-      //qsort((void *) data->allele[locus], maxalleles[locus], sizeof(char *), strcmp);
 #ifdef BEAGLE
       world->mutationmodels[world->sublocistarts[locus]].numstates = maxalleles[locus];
       world->mutationmodels[world->sublocistarts[locus]].basefreqs = (double *) calloc(world->mutationmodels[world->sublocistarts[locus]].numstates,sizeof(double));
@@ -1941,11 +1921,10 @@ void print_spectra(world_fmt * world, option_fmt * options,data_fmt * data)
       freq[pop] = (MYREAL **) mycalloc(data->loci, sizeof(MYREAL *));
       for (locus = 0; locus < data->loci; locus++)
 	{
-	  freq[pop][locus] = (MYREAL *) mycalloc(maxalleles[locus], sizeof(MYREAL));
+	  freq[pop][locus] = (MYREAL *) mycalloc(1+maxalleles[locus], sizeof(MYREAL));
 	}
     }
   
-
   // calculate spectrum
   for (pop1 = 0; pop1 < data->numpop; pop1++)
     {
@@ -1993,15 +1972,16 @@ void print_spectra(world_fmt * world, option_fmt * options,data_fmt * data)
 	}
     }
   // print in ascii
+  //fprintf(stdout,"Allele frequency spectra\n");
   fprintf(outfile,"Allele frequency spectra\n");
   fprintf(outfile,"========================\n\n");
   for (locus = 0; locus < data->loci; locus++)
     {
-      memset(maxallelepop,'\0',sizeof(long)*data->numpop);
       fprintf(outfile,"Locus %li\n", locus + 1);
       fprintf(outfile,"Allele  ");
       for (pop1 = 0; pop1 < data->numpop; pop1++)
 	{
+	  maxallelepop[pop1]=0;
 	  pop = options->newpops[pop1]-1;
 	  fprintf(outfile,"Pop%-2li  ",pop+1);
 	}
@@ -2055,6 +2035,7 @@ void print_spectra(world_fmt * world, option_fmt * options,data_fmt * data)
 	}
       myfree(freq[pop]);
     }
+  //printf("finished with printspectra");
   myfree(freq);
   myfree(maxalleles);
   myfree(maxallelepop);
@@ -2181,6 +2162,121 @@ print_header (FILE * outfile, long pop, world_fmt * world,
 }
 
 
+MYREAL findleastsquare(MYREAL *rawdata, long total, long repeatlength, long shift, MYREAL *startvalue)
+{
+  long i;
+  long j;
+  MYREAL lowvalue=MYREAL_MAX;
+  MYREAL highvalue=0.;
+  MYREAL sum = 0.0;
+  for(i=0;i<total;i++)
+    {
+      if(rawdata[i] < lowvalue)
+	lowvalue = rawdata[i];
+      if(rawdata[i]> highvalue)
+	highvalue = rawdata[i];
+    }
+  long high = (long) (highvalue+repeatlength+shift);
+  long low  = (long) (lowvalue -repeatlength+shift);
+  if(low<0){
+    low = 0;
+  }
+  *startvalue = low;
+  MYREAL value;
+  MYREAL oldvalue;
+  for(i=0;i<total;i++)
+    {
+      oldvalue = MYREAL_MAX;
+      for(j=low ; j < high; j += repeatlength)
+	{
+	  value = rawdata[i] - j;
+	  value *= value;
+	  if(value < oldvalue)
+	    {
+	      oldvalue = value;
+	    }
+	}
+      sum += oldvalue;
+    }
+  return sum;
+}
+
+void find_allele_repeatlength(data_fmt *data, option_fmt *options, long locus)
+{
+  long zz=0;
+  long pop;
+  long ind;
+  long r;
+  for (pop = 0; pop < data->numpop; pop++)
+    {
+      zz += data->numind[pop][locus];
+    }
+  MYREAL * rawdata = (MYREAL *) calloc(2 * zz, sizeof(MYREAL));
+  long z=0;
+  for (pop = 0; pop < data->numpop; pop++)
+    {
+      for (ind = 0; ind < data->numind[pop][locus]; ind++)
+	{
+	  char *a1 = data->yy[pop][ind][locus][0];
+	  char *a2 = data->yy[pop][ind][locus][1];
+	  if (a1[0]!='?')
+	    {
+	      rawdata[z++] = atof(a1);
+	    }
+	  if (a2[0]!='?')
+	    {
+	      rawdata[z++] = atof(a2);
+	    }
+	}
+    }
+  long total=z;
+  MYREAL minLS=MYREAL_MAX;
+  long keepr=0;
+  MYREAL value;
+  long intval;
+  MYREAL startvalue= 0.0;
+  MYREAL keepstartvalue= 0.0;
+  for(r=0;r<data->repeatlength[locus];r++)
+    {
+      //MYREAL endvalue  = 0.0;
+      MYREAL leastsquare=findleastsquare(rawdata,total,data->repeatlength[locus],r,&startvalue);
+      if(leastsquare < minLS)
+	{
+	  minLS = leastsquare;
+	  keepr = r;
+	  keepstartvalue = startvalue;
+	}
+    }
+  for (pop = 0; pop < data->numpop; pop++)
+    {
+      for (ind = 0; ind < data->numind[pop][locus]; ind++)
+	{
+	  char *a1 = data->yy[pop][ind][locus][0];
+	  char *a2 = data->yy[pop][ind][locus][1];
+	  if (a1[0]!='?')
+	    {
+	      value = atof(a1);
+	      //Floor[(279.47 - 259)/4 + If[(Random[]) < 1 - FractionalPart[(279.47 - 259)/4 ], 0, 1]], {1000}] // Tally
+	      MYREAL nonfloored = (value - keepstartvalue)/data->repeatlength[locus]; 
+	      MYREAL floored = floor(nonfloored);
+	      MYREAL diff = nonfloored - floored; 
+	      intval = (long) floored + (RANDUM() < (1.0-diff) ? 0 : 1); 
+	      sprintf(data->yy[pop][ind][locus][0],"%li",intval);
+	    }
+	  if (a2[0]!='?')
+	    {
+	      value = atof(a2);
+	      MYREAL nonfloored = (value - keepstartvalue)/data->repeatlength[locus]; 
+	      MYREAL floored = floor(nonfloored);
+	      MYREAL diff = nonfloored - floored; 
+	      intval = (long) floored + (RANDUM() < (1.0-diff) ? 0 : 1); 
+	      //	      intval = (long) lround((value + keepr - keepstartvalue)/data->repeatlength[locus]);
+	      sprintf(data->yy[pop][ind][locus][1],"%li",intval);
+	    }
+	}
+    }
+}
+
 
 void
 create_alleles (data_fmt * data, option_fmt *options)
@@ -2194,8 +2290,10 @@ create_alleles (data_fmt * data, option_fmt *options)
     char a2[DEFAULT_ALLELENMLENGTH];
     for (locus = 0; locus < data->loci; locus++)
     {
-        z = 0;
-        for (pop = 0; pop < data->numpop; pop++)
+      if(data->repeatlength[locus]!=0)
+	find_allele_repeatlength(data, options,locus);
+      z = 0;
+      for (pop = 0; pop < data->numpop; pop++)
         {
             for (ind = 0; ind < data->numind[pop][locus]; ind++)
             {
@@ -2415,6 +2513,8 @@ read_date_fromfile (FILE * datefile, data_fmt *data, option_fmt *options, long n
 		    data->sampledates[pop][locus][ind].date = temp;
 		    data->sampledates[pop][locus][ind].name = (char *) mycalloc(strlen(name)+1,sizeof(char));
 		    strcpy(data->sampledates[pop][locus][ind].name, name);
+		    unpad(data->sampledates[pop][locus][ind].name," ");
+		    translate(data->sampledates[pop][locus][ind].name,' ', '_');
 		    if(oldest < temp)
 		      oldest = temp;
 		    if(youngest > temp)
