@@ -19,16 +19,16 @@ Copyright 2002 Peter Beerli and Joseph Felsenstein
 Copyright 2003-2008 Peter Beerli
 Copyright 2009-2010 Peter Beerli and Michal Palczewski
  
-$Id: definitions.h 1714 2010-09-05 03:21:45Z beerli $
+$Id: definitions.h 1742 2010-10-24 17:03:07Z beerli $
  
   *----------------------------------------------------------------
   */
 #ifndef MIGRATEVERSION
-#define MIGRATEVERSION "3.1.10"
+#define MIGRATEVERSION "3.2.1"
 #else
 #ifdef SGI
 #undef MIGRATEVERSION
-#define MIGRATEVERSION "3.1.10"
+#define MIGRATEVERSION "3.2.1"
 #endif
 #endif
 #ifndef MIGRATESUBVERSION
@@ -60,7 +60,8 @@ $Id: definitions.h 1714 2010-09-05 03:21:45Z beerli $
 #endif
 
 #define FGETS  myfgets
-
+#define ZNZFGETS  myznzfgets
+#define FGETS2  myfgetssafe
 #ifdef MPI
 #include <mpi.h>
 #define FPRINTF mpi_fprintf
@@ -152,7 +153,7 @@ $Id: definitions.h 1714 2010-09-05 03:21:45Z beerli $
 #define AICFILENUM 19
 #define BAYESFILE "bayesfile"
 #define BAYESFILENUM 20
-#define BAYESMDIMFILE "bayesallfile"
+#define BAYESMDIMFILE "bayesallfile.gz"
 #define BAYESMDIMFILENUM 22
 #define PDFOUTFILE "outfile.pdf"
 #define PDFOUTFILENUM 21
@@ -237,6 +238,7 @@ $Id: definitions.h 1714 2010-09-05 03:21:45Z beerli $
 
 //#include "sighandler.h"
 #define FClose(file) if (file) fclose(file) ; file=NULL
+#define znzClose(file) if (file) znzclose(file) ; file=NULL
 #ifdef WIN32
 #ifndef WINDOWS
 #define WINDOWS
@@ -313,8 +315,9 @@ typedef int boolean;
    observed alleles, much bigger does not harm */
 #define MAXALLELES            93L
 #define SMALLBUFSIZE    255L
+#define ONEMEGABYTE   1024000
 #define MAXBUFSIZE  10000000L
-#define LINESIZE      100000L /* setting this smaller can break */
+#define LINESIZE        1024L /* setting this smaller can break */
 #define LONGLINESIZE  100000L /* setting this smaller can break */
 #define SUPERLINESIZE 100000L /*used to read many many loci-sites and indiv numbers*/ 
 #define STRSIZE              255L /* setting this smaller can break */
@@ -346,7 +349,7 @@ typedef int boolean;
 so that the given number of
 long chains is used */
 #define CHAINVARIANCEDELTA 0.01 /*difference of variances of parts of a chain [see burnin_autostop]*/
-#define ESSMINIMUM 100  /*effective sample size minimum see burnin_autostop*/
+#define ESSMINIMUM 200  /*effective sample size minimum see burnin_autostop*/
 #define GELMAN_MYSTIC_VALUE 1.2
 #define PLUSCHAIN 10
 #ifndef MAXLONG
