@@ -18,7 +18,7 @@ Copyright 2003-2006 Peter Beerli, Tallahassee FL
  and is copyrighted. Of course, we do not guarantee that the software
  works and are not responsible for any damage you may cause or have.
  
-$Id: mcmc1.c 1691 2010-05-14 14:26:56Z beerli $
+$Id: mcmc1.c 1727 2010-10-18 15:04:46Z beerli $
 -------------------------------------------------------*/
 /* \file mcmc1.c
 
@@ -55,9 +55,7 @@ boolean debugtip;
 extern int myID;
 extern MYREAL probg_treetimesX(world_fmt *world, vtlist *tl, long T);
 
-/* global for this file */
-proposal_fmt * gproposal;
-
+extern void     zero_xseq(xarray_fmt *x, long sites, long categs);
 
 /* prototypes ------------------------------------------- */
 // metropolize over trees
@@ -102,6 +100,8 @@ long migration_from (long to, proposal_fmt * proposal);
 
 MYREAL prob_tree (world_fmt * world, timelist_fmt * tyme);
 void traverse_check(node *theNode);
+void reset_proposal (proposal_fmt ** proposal, world_fmt *world);
+
 
 /* Functions ++++++++++++++++++++++++++++++++++++++++++++++++*/
 void
@@ -131,10 +131,7 @@ set_tree_dirty (node * p)
 long
 tree_update (world_fmt * world, long g)
 {    /*return 1 if tree was accepted, 0 otherwise */
-    static long treefilepos; /* write position in the treefile */
-#ifdef TESTING2
-    static boolean have_proposal=FALSE;
-#endif
+  //    static long treefilepos; /* write position in the treefile */
 #ifdef BEAGLE
     int beagle_control=1;
 #endif
@@ -150,10 +147,14 @@ tree_update (world_fmt * world, long g)
 
     boolean uepsuccess = FALSE;
 #endif
-
-    proposal_fmt *proposal; /*scratchpad  in which all involved nodes
-               are recorded and help arrays, e.g. migration arrays,
-               are stored */
+#ifdef TESTING2
+    proposal_fmt *proposal = world->proposal; 
+#else
+    proposal_fmt *proposal=NULL; 
+#endif
+    /*scratchpad  in which all involved nodes
+      are recorded and help arrays, e.g. migration arrays,
+      are stored */
     timelist_fmt *timevector = NULL; /* local timelist */
     vtlist *tentry = NULL; /*pointer into timeslice */
 
@@ -166,13 +167,12 @@ tree_update (world_fmt * world, long g)
 #endif
     new_localtimelist (&timevector, &world->treetimes[0], world->numpop);
 #ifdef TESTING2
-    if(have_proposal)
-      set_new_proposal (&proposal, &world->treetimes[0], world);
+    if(world->has_proposal_details)
+      reset_proposal (&proposal, world);//, &world->treetimes[0], world);
     else
       {
-    	new_proposal (&gproposal, &world->treetimes[0], world);
     	set_new_proposal (&proposal, &world->treetimes[0], world);
-    	have_proposal = TRUE;
+    	world->has_proposal_details = TRUE;
       }
 #else
     new_proposal (&proposal, &world->treetimes[0], world);
@@ -381,8 +381,8 @@ tree_update (world_fmt * world, long g)
 #endif /*UEP*/
         /* create a new timelist */
         construct_tymelist (world, &world->treetimes[0]);
-        if (world->options->treeprint != _NONE && world->cold)
-	  print_tree (world, g, &treefilepos);
+	//        if (world->options->treeprint != _NONE && world->cold)
+	//  print_tree (world, g, &treefilepos);
         world->migration_counts = 0;
         /* report the number of migration on the tree */
         count_migrations (world->root->next->back, &world->migration_counts);
@@ -503,23 +503,11 @@ new_proposal (proposal_fmt ** proposal, timelist_fmt * tl, world_fmt * world)
     // node data holding vector for bordernodes, line_f, line_t
     (*proposal)->nodedata =
         (node **) mycalloc (3 * listsize, sizeof (node *));
-    // adjacent nodes of the pcked nodes
+    // adjacent nodes of the picked nodes
     (*proposal)->bordernodes = (*proposal)->nodedata;
     // line_f nodes
     (*proposal)->line_f =   (*proposal)->bordernodes + listsize;
     (*proposal)->line_t =  (*proposal)->line_f + listsize;
-#ifdef checkdebugstuff
-    // nodes above the picked node
-    (*proposal)->aboveorigin = (node **) mycalloc (listsize, sizeof (node *));
-    // node data holding vector for aboveorigin, bordernodes, line_f, line_t
-    //    (*proposal)->nodedata =
-    //    (node **) mycalloc (2 * listsize + 2 * sumtips, sizeof (node *));
-    // adjacent nodes of the pcked nodes
-    (*proposal)->bordernodes = (node **) mycalloc (listsize, sizeof (node *));
-    // line_f nodes
-    (*proposal)->line_f =   (node **) mycalloc (listsize, sizeof (node *));;
-    (*proposal)->line_t =   (node **) mycalloc (listsize, sizeof (node *));;
-#endif
     // mf holds also mt array  
     (*proposal)->mf = (MYREAL *) mycalloc (2* (*proposal)->endsite, sizeof (MYREAL));
     (*proposal)->mt = (*proposal)->mf + (*proposal)->endsite;
@@ -572,23 +560,20 @@ new_proposal (proposal_fmt ** proposal, timelist_fmt * tl, world_fmt * world)
 }
 
 ///
-/// sets new proposal structure using template gproposal, thins function will replace new_proposal()
+/// sets new proposal structure using template gproposal, this function will be called most of the time instead of new_proposal()
 void
 set_new_proposal (proposal_fmt ** proposal, timelist_fmt * tl, world_fmt * world)
 {
+  long mal = world->data->maxalleles[world->locus];
   long oldsize=0;
   long newsize =0;
-  long sumtips ;
+  long sumtips = world->sumtips;
 #ifdef UEP    
     long j;
 #endif
-    //    long mal = world->data->maxalleles[world->locus];
-    long listsize = ((*tl).allocT + (*tl).T + 5);
-    //    long sumtips = world->sumtips;
-    //could be used to calculate p(gn|go):    (*proposal)->nu = 0.;
-    // allocate the scratchpad (contains a shadow of the tree)
-    (*proposal) = gproposal;
-    sumtips = (*proposal)->sumtips;
+    long listsize = 2*(world->sumtips + 2);
+    (*proposal)->likelihood = -HUGE;
+    (*proposal)->sumtips = sumtips;
     // pointers and values to outside structures
     (*proposal)->world = world;
     (*proposal)->datatype = world->options->datatype;
@@ -602,56 +587,52 @@ set_new_proposal (proposal_fmt ** proposal, timelist_fmt * tl, world_fmt * world
     // precalculated values
     (*proposal)->mig0list = world->mig0list;
     (*proposal)->design0list = world->design0list;
-    if(listsize > (*proposal)->listsize)
-      {
-	newsize = 2 * listsize + 2 * sumtips;
-	(*proposal)->nodedata = (node **) myrealloc ((*proposal)->nodedata, newsize * sizeof (node *));
-	oldsize = (2*((*proposal)->listsize)+2*sumtips);
-	memset((*proposal)->nodedata+oldsize, 0, (newsize-oldsize) * sizeof (node *));
-	(*proposal)->aboveorigin = (*proposal)->nodedata;
-	(*proposal)->bordernodes = (*proposal)->nodedata + listsize;
-	(*proposal)->line_f =   (*proposal)->nodedata + listsize + sumtips;
-	(*proposal)->line_t =  (*proposal)->nodedata + listsize + 2 * sumtips;
-
-      } 
+    newsize = 4 * listsize;
+    if((*proposal)->nodedata == NULL)
+      (*proposal)->nodedata = (node **) calloc (newsize, sizeof (node *));
+    else
+      (*proposal)->nodedata = (node **) realloc ((*proposal)->nodedata, newsize * sizeof (node *));
+    oldsize = (*proposal)->listsize;
+    memset((*proposal)->nodedata, 0, newsize * sizeof (node *));
+    (*proposal)->aboveorigin = (*proposal)->nodedata;
+    (*proposal)->bordernodes = (*proposal)->nodedata + listsize;
+    (*proposal)->line_f =   (*proposal)->bordernodes + listsize;
+    (*proposal)->line_t =  (*proposal)->line_f + listsize;
     (*proposal)->listsize =  listsize;
-
-
     // mf holds also mt array  
-    //(*proposal)->mf = (MYREAL *) mycalloc (2* (*proposal)->endsite, sizeof (MYREAL));
-    //(*proposal)->mt = (*proposal)->mf + (*proposal)->endsite;
-
-    //if (strchr (SEQUENCETYPES, (*proposal)->datatype))
-    //{
-    //    allocate_xseq(&(*proposal)->xf, world->data->seq->endsite, world->options->rcategs);
-    //    allocate_xseq(&(*proposal)->xt, world->data->seq->endsite, world->options->rcategs);
-    //}
-    //else
-    //{
-    //    (*proposal)->xf.a = (MYREAL *) mycalloc (1, sizeof (MYREAL) * mal);
-    //    (*proposal)->xt.a = (MYREAL *) mycalloc (1, sizeof (MYREAL) * mal);
-    //    }
-//(*proposal)->old_migr_table_counter = 4 * sumtips /* 100 */ ;
-//    (*proposal)->old_migr_table_counter2 = 4 * sumtips /* 100 */ ;
-//(*proposal)->migr_table =
-//      (migr_table_fmt *) mycalloc (1,
-//                                 sizeof (migr_table_fmt) *
-//                                 (*proposal)->old_migr_table_counter);
-//  (*proposal)->migr_table2 =
-//      (migr_table_fmt *) mycalloc ((*proposal)->old_migr_table_counter2,
-//                                   sizeof (migr_table_fmt));
+    (*proposal)->mf = (MYREAL *) mycalloc (2* (*proposal)->endsite, sizeof (MYREAL));
+    (*proposal)->mt = (*proposal)->mf + (*proposal)->endsite;
+    if (strchr (SEQUENCETYPES, (*proposal)->datatype))
+    {
+      allocate_xseq(&(*proposal)->xf, world->data->seq[0]->endsite, world->options->rcategs);
+      allocate_xseq(&(*proposal)->xt, world->data->seq[0]->endsite, world->options->rcategs);
+    }
+    else
+    {
+      (*proposal)->xf.a = (MYREAL *) mycalloc (1, sizeof (MYREAL) * mal);
+      (*proposal)->xt.a = (MYREAL *) mycalloc (1, sizeof (MYREAL) * mal);
+    }
+    (*proposal)->old_migr_table_counter = 4 * sumtips /* 100 */ ;
+    (*proposal)->old_migr_table_counter2 = 4 * sumtips /* 100 */ ;
+    (*proposal)->migr_table =
+      (migr_table_fmt *) mycalloc (1,
+				   sizeof (migr_table_fmt) *
+				   (*proposal)->old_migr_table_counter);
+    (*proposal)->migr_table2 =
+      (migr_table_fmt *) mycalloc ((*proposal)->old_migr_table_counter2,
+                                   sizeof (migr_table_fmt));
     (*proposal)->migr_table_counter = 0;
     (*proposal)->migr_table_counter2 = 0;
 #ifdef UEP
     if (world->options->uep)
     {
-//        (*proposal)->ueplike =
-//            (MYREAL **) mycalloc (world->data->uepsites, sizeof (MYREAL *));
-//        (*proposal)->ueplike[0] =
-//            (MYREAL *) mycalloc (world->numpop * world->data->uepsites,
-//                               sizeof (MYREAL));
-//        for (j = 1; j < world->data->uepsites; ++j)
-//          (*proposal)->ueplike[j] = (*proposal)->ueplike[0] + j * world->numpop;
+        (*proposal)->ueplike =
+            (MYREAL **) mycalloc (world->data->uepsites, sizeof (MYREAL *));
+        (*proposal)->ueplike[0] =
+            (MYREAL *) mycalloc (world->numpop * world->data->uepsites,
+                               sizeof (MYREAL));
+        for (j = 1; j < world->data->uepsites; ++j)
+          (*proposal)->ueplike[j] = (*proposal)->ueplike[0] + j * world->numpop;
 
         (*proposal)->ut.s = (pair *) mycalloc (world->data->uepsites, sizeof (pair));
         (*proposal)->uf.s = (pair *) mycalloc (world->data->uepsites, sizeof (pair));
@@ -659,6 +640,13 @@ set_new_proposal (proposal_fmt ** proposal, timelist_fmt * tl, world_fmt * world
         (*proposal)->umf = (MYREAL *) mycalloc (world->data->uepsites, sizeof (MYREAL));
     }
 #endif
+
+#ifdef BEAGLE
+    (*proposal)->leftid   = 0;
+    (*proposal)->rightid  = 0;
+    (*proposal)->parentid = 0;
+    reset_beagle(world->beagle);
+#endif
 }
 
 
@@ -1503,45 +1491,105 @@ pre_population (proposal_fmt * proposal, vtlist * ltime, long gte,
 
 #ifdef TESTING2
 ///
-/// freeing the proposal structure, this will be replaced by a function that resets permanently
-/// allocated structure [reset_proposal()]
-void
-free_proposal (proposal_fmt * proposal)
+/// freeing the proposal structure, this is the replacement of the real free_proposal method
+/// and does not allocate, but reuses old memory
+void free_proposal(proposal_fmt *proposal)
 {
-  //  long j;
-  long listsize = proposal->listsize;
-  long sumtips = proposal->sumtips;
-  long smax = proposal->world->data->maxalleles[proposal->world->locus];
+  // do nothing
+}
+void reset_simple_proposal_variables(proposal_fmt **proposal)
+{
+  (*proposal)->mig_removed = FALSE;
+  (*proposal)->rr = 0.0;
+  (*proposal)->origin = NULL;
+  (*proposal)->target = NULL;
+  (*proposal)->realtarget = NULL;
+  (*proposal)->tsister = NULL;
+  (*proposal)->realtsister = NULL;
+  (*proposal)->osister = NULL;
+  (*proposal)->realosister = NULL;
+  (*proposal)->ocousin = NULL;
+  (*proposal)->realocousin = NULL;
+  (*proposal)->oback = NULL;
+  (*proposal)->realoback = NULL;
+  //
+  (*proposal)->connect = NULL;
+  (*proposal)->likelihood = 0.0;
+  (*proposal)->time = 0.0;
+  (*proposal)->v = 0.0;
+  (*proposal)->vs = 0.0;
+  //
+#ifdef UEP
+  (*proposal)->ueplikelihood = 0.0;
+#endif
+  (*proposal)->migr_table_counter=0;
+  (*proposal)->migr_table_counter2=0;
+  (*proposal)->timeslice = 0;
+  //
+  (*proposal)->treelen = 0.0;
+#ifdef BEAGLE
+  (*proposal)->parentid = 0;
+  (*proposal)->leftid = 0;
+  (*proposal)->rightid = 0;
+#endif
+}
+
+void
+reset_proposal (proposal_fmt ** proposal, world_fmt *world)
+{
+  const long listsize = 2 * (world->sumtips + 2);
+  const long newsize = 4 * listsize;
+  const long mal = (*proposal)->world->data->maxalleles[(*proposal)->world->locus];
+  (*proposal)->likelihood = -HUGE;
+  // pointers and values to outside structures
+  (*proposal)->world = world;
+  (*proposal)->datatype = world->options->datatype;
+  (*proposal)->sumtips = world->sumtips;
+  (*proposal)->numpop = world->numpop;
+  (*proposal)->endsite = world->data->seq[0]->endsite;
+  (*proposal)->fracchange = world->data->seq[0]->fracchange;
+  (*proposal)->param0 = world->param0;
+  (*proposal)->root = world->root;
+  (*proposal)->migration_model = world->options->migration_model;
+  // precalculated values
+  (*proposal)->mig0list = world->mig0list;
+  (*proposal)->design0list = world->design0list;
+  (*proposal)->listsize =  listsize;
   // ..line_t are also reset with this
-  memset(proposal->nodedata,0,sizeof(node *) * (2 * listsize + 2 * sumtips)); 
-  memset(proposal->mf,0, sizeof(MYREAL)*(2 * proposal->endsite)); // proposal->mt is also freed with this
-  if (strchr (SEQUENCETYPES, proposal->datatype))
+  long i;
+  for(i=0;i<newsize;i++)
+    (*proposal)->nodedata[i]=NULL;
+  //  memset((*proposal)->nodedata,0,sizeof(node *) * newsize); 
+  memset((*proposal)->mf,0, sizeof(MYREAL)*(2 * (*proposal)->endsite)); // (*proposal)->mt is also freed with this
+  // resetting pointers
+  reset_simple_proposal_variables(proposal);
+
+  if (strchr (SEQUENCETYPES, (*proposal)->datatype))
     {
-      zero_xseq(&proposal->xf,proposal->world->data->seq->endsite, proposal->world->options->rcategs);
-      zero_xseq(&proposal->xt,proposal->world->data->seq->endsite, proposal->world->options->rcategs);
+      zero_xseq(&(*proposal)->xf,(*proposal)->world->data->seq[0]->endsite, (*proposal)->world->options->rcategs);
+      zero_xseq(&(*proposal)->xt,(*proposal)->world->data->seq[0]->endsite, (*proposal)->world->options->rcategs);
     }
     else
     {
-        memset(proposal->xf.a, 0, smax);
-        memset(proposal->xt.a, 0, smax);
+        memset((*proposal)->xf.a, 0, mal);
+        memset((*proposal)->xt.a, 0, mal);
     }
-    memset(proposal->migr_table, 0, sizeof(migr_table_fmt) * proposal->old_migr_table_counter);
-    memset(proposal->migr_table2, 0, sizeof(migr_table_fmt) * proposal->old_migr_table_counter2);
+    memset((*proposal)->migr_table, 0, sizeof(migr_table_fmt) * (*proposal)->old_migr_table_counter);
+    memset((*proposal)->migr_table2, 0, sizeof(migr_table_fmt) * (*proposal)->old_migr_table_counter2);
 #ifdef UEP
 
-    if (proposal->world->options->uep)
+    if ((*proposal)->world->options->uep)
     {
-        memset(proposal->ueplike, 0, sizeof(MYREAL) * world->numpop * world->data->uepsites);
+        memset((*proposal)->ueplike, 0, sizeof(MYREAL) * world->numpop * world->data->uepsites);
         for (j = 1; j < world->data->uepsites; ++j)
-	  (*proposal)->ueplike[j] = (*proposal)->ueplike[0] + j * world->numpop;
+	  (*(*proposal))->ueplike[j] = (*(*proposal))->ueplike[0] + j * world->numpop;
 
-        memset(proposal->uf.s, 0, world->data->uepsites * sizeof(pair));
-        memset(proposal->ut.s, 0, world->data->uepsites * sizeof(pair));
-        memset(proposal->umf, 0, world->data->uepsites * sizeof(MYREAL));
-        memset(proposal->umt, 0, world->data->uepsites * sizeof(MYREAL));
+        memset((*proposal)->uf.s, 0, world->data->uepsites * sizeof(pair));
+        memset((*proposal)->ut.s, 0, world->data->uepsites * sizeof(pair));
+        memset((*proposal)->umf, 0, world->data->uepsites * sizeof(MYREAL));
+        memset((*proposal)->umt, 0, world->data->uepsites * sizeof(MYREAL));
     }
 #endif
-    //    myfree(proposal);
 }
 #else
 ///
