@@ -16,7 +16,7 @@ Copyright 2003-2007 Peter Beerli, Tallahassee FL
  works and are not responsible for any damage you may cause or have.
  
  
-$Id: reporter.c 1716 2010-09-07 20:33:27Z beerli $
+$Id: reporter.c 1722 2010-10-09 15:27:25Z beerli $
  
 -------------------------------------------------------*/
 /* \file reporter.c 
@@ -345,7 +345,7 @@ void collect_ess_values(world_fmt *world)
 void print_bayes_ess(FILE *file, world_fmt *world, long numparam, int offset, 
 			  MYREAL *autocorr, MYREAL *effsample)
 {
-  long pa;
+  long pa=0;
   long pa0;
   long numpop = world->numpop;
   long numpop2 = world->numpop2;
@@ -374,10 +374,19 @@ void print_bayes_ess(FILE *file, world_fmt *world, long numparam, int offset,
 	      if(pa0 < numpop)
 		{
 		  if((pa0 == pa) && (custm2[pa0] == '*'))
-		    pos += sprintf(buffer+pos,"%*.*s%s_%li               % -5.5f           %10.2f\n", offset, offset, " ", "Theta",pa0+1,autocorr[pa],effsample[pa]);  
+		    {
+		      pos += sprintf(buffer+pos,"%*.*s%s_%li               % -5.5f           %10.2f\n", 
+				     offset, offset, " ", "Theta",pa0+1,autocorr[pa],effsample[pa]); 
+		      if(effsample[pa]<ESSMINIMUM && file==world->outfile)
+			record_warnings(world,"Param %li: Effective sample size of run seems too short! ",pa+1);
+		    }
 		  else
-		    pos += sprintf(buffer+pos,"%*.*s%s_%li=%li [%c]      % -5.5f        %10.2f\n", offset, offset, " ", 
-				   "Theta",pa0+1, pa+1, custm2[pa0],autocorr[pa],effsample[pa]);  
+		    {
+		      pos += sprintf(buffer+pos,"%*.*s%s_%li=%li [%c]      % -5.5f        %10.2f\n", offset, offset, " ", 
+				     "Theta",pa0+1, pa+1, custm2[pa0],autocorr[pa],effsample[pa]);  
+		      if(effsample[pa]<ESSMINIMUM && file==world->outfile)
+			record_warnings(world,"Param %li: Effective sample size of run seems too short! ",pa+1);
+		    }
 		}
 	      else
 		{
@@ -385,11 +394,18 @@ void print_bayes_ess(FILE *file, world_fmt *world, long numparam, int offset,
 		  if((pa0==pa) && (custm2[pa0]=='*'))
 		    {
 		      pos += sprintf(buffer+pos,"%*.*s%s_%li->%li                % -5.5f           %10.2f\n", offset, offset, " ", "M", frompop+1, topop+1,autocorr[pa],effsample[pa]);
+		      if(effsample[pa]<ESSMINIMUM && file==world->outfile)
+			record_warnings(world,"Param %li: Effective sample size of run seems too short! ",pa+1);
+
 		    }
 		  else
 		    {
 		      m2mm(pa,numpop,&frompop2,&topop2);
-		      pos += sprintf(buffer+pos,"%*.*s%s_(%li,%li) [%c]             % -5.5f           %10.2f\n", offset, offset, " ", "M", frompop+1, topop+1, custm2[pa0], autocorr[pa], effsample[pa]);
+		      pos += sprintf(buffer+pos,"%*.*s%s_(%li,%li) [%c]             % -5.5f           %10.2f\n", 
+				     offset, offset, " ", "M", frompop+1, topop+1, custm2[pa0], autocorr[pa], effsample[pa]);
+		      if(effsample[pa]<ESSMINIMUM && file==world->outfile)
+			record_warnings(world,"Param %li: Effective sample size of run seems too short! ",pa+1);
+
 		    }
 		}
 	    }
@@ -399,12 +415,17 @@ void print_bayes_ess(FILE *file, world_fmt *world, long numparam, int offset,
 	  if(pa0+1 == numparam)
 	    {
 	      pos += sprintf(buffer+pos,"%*.*s%s         % -5.5f           %10.2f\n", offset, offset, " ", "Ln[Prob(D|P)]",autocorr[pa0],effsample[pa0]);
+	      if(effsample[pa]<ESSMINIMUM && file==world->outfile)
+		record_warnings(world,"Param %li: Effective sample size of run seems too short! ",pa0+1);		      
 	    }
 	  else
 	    {
 	      if(world->locus + world->numpop2 == pa0)
 		{
-		  pos += sprintf(buffer+pos,"%*.*s%s_%-5li            % -5.5f           %10.2f\n", offset, offset, " ", "Rate", world->locus, autocorr[pa0],effsample[pa0]);
+		  pos += sprintf(buffer+pos,"%*.*s%s_%-5li            % -5.5f           %10.2f\n", 
+				 offset, offset, " ", "Rate", world->locus, autocorr[pa0],effsample[pa0]);
+		  if(effsample[pa]<ESSMINIMUM && file==world->outfile)
+		    record_warnings(world,"Param %li: Effective sample size of run seems too short! ",pa0+1);		      		  
 		}
 	    }
 	}
