@@ -16,7 +16,7 @@
  works and are not responsible for any damage you may cause or have.
  
  
- $Id: sequence.c 1628 2010-02-05 21:16:08Z beerli $
+ $Id: sequence.c 1732 2010-10-19 15:32:31Z beerli $
  
 -------------------------------------------------------*/
 /* \file sequence.c
@@ -665,7 +665,7 @@ makevalues_seq (world_fmt * world, option_fmt * options, data_fmt * data,
 	  if (!options->usertree)
 	    {
 	      allocate_tip (world, options, &(treenode[z]), pop, locus, z, ind,
-			    data->indnames[pop][ind][locus]);
+			    data->indnames[pop][ind]);
 	      //treenode[z]->tyme = find_tipdate(treenode[z]->nayme, pop,world);
 	      world->data->sampledates[pop][locus][ind].id = treenode[z]->id;
 	      z++;
