@@ -22,12 +22,14 @@
  and is copyrighted. Of course, we do not guarantee that the software
  works and are not responsible for any damage you may cause or have.
  
-   $Id: sighandler.c 1549 2009-06-04 20:06:57Z beerli $                  
+   $Id: sighandler.c 1737 2010-10-21 16:10:07Z beerli $                  
    ----------------------------------------------------- */
 /*! \file sighandler.c */
 #include "sighandler.h"
 #include "migrate_mpi.h"
-
+#ifdef ZNZ
+#include "znzlib.h"
+#endif
 #ifdef DMALLOC_FUNC_CHECK
 #include <dmalloc.h>
 #endif
@@ -285,6 +287,44 @@ sig_error (char string[], char filename[], long line)
     exit (EXIT_FAILURE);
 }
 
+///
+/// replacement for fgets so that migrate can read 
+/// files from different computers with different line-endings
+/// should be able to read a file with
+/// - windows
+/// - mac
+/// - unix
+/// line-endings.
+#ifdef ZNZ
+char 
+myznzfgets (char *buffer, long bufsize, znzFile infile)
+{
+  long count = 0;
+  char ch = '\0';
+  char oldch;
+  bufsize--;
+  ch = znzgetc (infile);
+  while (!(ch == '\r' || ch == '\n' || ch == EOF) && count < bufsize)
+    {
+      buffer[count] = ch;
+      count++;
+      ch = znzgetc (infile);
+    }
+  if (ch == '\r')
+    {
+      oldch = ch;
+      ch = znzgetc (infile);
+      if(!((ch == EOF) || (ch == '\n')))
+	{
+	  znzungetc (ch, infile);
+	  buffer[count] = '\0';
+	   return oldch;
+	}
+    }
+  buffer[count] = '\0';
+  return ch;
+}
+#endif
 ///
 /// replacement for fgets so that migrate can read 
 /// files from different computers with different line-endings
@@ -323,6 +363,51 @@ myfgets (char *buffer, long bufsize, FILE * infile)
   return ch;
 }
 
+///
+/// replacement for fgets that reallocates if things are longer than buffer
+//  so that migrate can read 
+/// files from different computers with different line-endings
+/// should be able to read a file with
+/// - windows
+/// - mac
+/// - unix
+/// line-endings.
+char 
+myfgetssafe (char **buffer, long *bufsize, FILE * infile)
+{
+  long count = 0;
+  char ch = '\0';
+  char oldch;
+  long bs = *bufsize;
+  bs--;
+  ch = fgetc (infile);
+  while (!(ch == '\r' || ch == '\n' || ch == EOF) && count < bs)
+    {
+      (*buffer)[count] = ch;
+      count++;
+      if(count>=bs)
+	{
+	  bs += STRSIZE;
+	  *bufsize += STRSIZE;
+	  (*buffer) = (char *) realloc(*buffer, sizeof(char) * *bufsize);
+	}
+      ch = fgetc (infile);
+    }
+  if (ch == '\r')
+    {
+      oldch = ch;
+      ch = fgetc (infile);
+      if(!((ch == EOF) || (ch == '\n')))
+	{
+	  ungetc (ch, infile);
+	  (*buffer)[count] = '\0';
+	   return oldch;
+	}
+    }
+  (*buffer)[count] = '\0';
+  return ch;
+}
+
 #ifdef NEXTAPP
 void
 malloc_error_found (int value)
