@@ -34,10 +34,22 @@ extern void usererror (char string[], ...);
 extern void sig_error (char string[], char filename[], long line);
 
 extern char myfgets (char *buffer, long bufsize, FILE * infile);
-
+extern char myfgetssafe (char **buffer, long *bufsize, FILE * infile);
+#ifdef ZNZ
+#include "znzlib.h"
+extern char myznzfgets (char *buffer, long bufsize, znzFile infile);
+#endif
 #ifdef NEXTAPP
 extern void malloc_error_found (int value);
 #endif
+
+#ifdef MEMDEBUG
+extern void memdebug_free(void *ptr, const char file[], const long line);
+#define myfree(a) memdebug_free(a,__FILE__,__LINE__);
+#else
+#define myfree(a)  free(a)
+#endif
+
 #ifdef LAMARC_MALLOC
 extern void *LAMARC_malloc (size_t size, const char file[],
                                 const long line);
@@ -57,18 +69,22 @@ extern void *LAMARC_realloc (void *ptr, size_t,
 #ifdef MPI
 #ifdef DEBUG_MPI
 #define MYMPISEND(a,b,c,d,e,f) printf("%i>before ssend of %li bytes to %i with tag %i (%s,%i)\n",myID, b* sizeof(c), d, e,__FILE__,__LINE__);fflush(stdout);MPI_Ssend(a,b,c, d,  e,f);printf("%i>after send (%s,%i)\n",myID, __FILE__,__LINE__);fflush(stdout)
-#define MYMPIISEND(a,b,c,d,e,f,g) printf("%i>before isend of %li bytes to %i with tag %i in %i (%s,%i)\n",myID, b* sizeof(c), d, e, f, __FILE__,__LINE__);fflush(stdout);MPI_Isend(a,b,c,d,  e,f,g);printf("%i>after send (%s,%i)\n",myID, __FILE__,__LINE__);fflush(stdout)
-#define MYMPIRECV(a,b,c,d,e,f,g) printf("%i>before receive of %li bytes from %i with tag %i in %i (%s,%i)\n",myID, b* sizeof(c),d,e, f, __FILE__,__LINE__);fflush(stdout);MPI_Recv(a,b,c,d,e,f,g);printf("%i>after receive (%s,%i)\n",myID, __FILE__,__LINE__);fflush(stdout)
+//#define MYMPIISEND(a,b,c,d,e,f,g) printf("%i>before isend of %li bytes to %i with tag %i in %i (%s,%i)\n",myID, b* sizeof(c), d, e, (int) f, __FILE__,__LINE__);fflush(stdout);MPI_Isend(a,b,c,d,  e,f,g);printf("%i>after send (%s,%i)\n",myID, __FILE__,__LINE__);fflush(stdout)
+#define MYMPIISEND(a,b,c,d,e,f,g) printf("%i>before isend of %li bytes to %i with tag %i in %i (%s,%i)\n",myID, b* sizeof(c), d, e, (int) f, __FILE__,__LINE__);fflush(stdout);MPI_Send(a,b,c,d,  e,f);printf("%i>after send (%s,%i)\n",myID, __FILE__,__LINE__);fflush(stdout)
+#define MYMPIRECV(a,b,c,d,e,f,g) printf("%i>before receive of %li bytes from %i with tag %i in %i (%s,%i)\n",myID, b * sizeof(c),d,e, (int) f, __FILE__,__LINE__);fflush(stdout);MPI_Recv(a,b,c,d,e,f,g);printf("%i>after receive (%s,%i)\n",myID, __FILE__,__LINE__);fflush(stdout)
 #define MYMPIBCAST(a,b,c,d,e) printf("%i>before broadcast of %li bytes (%s,%i)\n",myID, ((long) b) * ((long) sizeof(c)), __FILE__,__LINE__);fflush(stdout);MPI_Bcast(a,b,c,d,e);printf("%i>after broadcast (%s,%i)\n",myID, __FILE__,__LINE__);fflush(stdout)
 #define MYMPIBARRIER(a) printf("%i>before barrier (%s,%i)\n",myID, __FILE__,__LINE__);fflush(stdout);MPI_Barrier(a);printf("%i>after barrier (%s,%i)\n",myID, __FILE__,__LINE__);fflush(stdout)
-#define MYMPIWAITALL(a,b,c) printf("%i>before waitall (%s,%i)\n",myID, __FILE__,__LINE__);fflush(stdout);MPI_Waitall(a,b,c);printf("%i>after waitall (%s,%i)\n",myID, __FILE__,__LINE__);fflush(stdout)
+//#define MYMPIWAITALL(a,b,c) printf("%i>before waitall (%s,%i)\n",myID, __FILE__,__LINE__);fflush(stdout);MPI_Waitall(a,b,c);printf("%i>after waitall (%s,%i)\n",myID, __FILE__,__LINE__);fflush(stdout)
+#define MYMPIWAITALL(a,b,c) /*do nothing*/
 #else
 #define MYMPISEND(a,b,c,d,e,f) MPI_Send(a,b,c,d,e,f)
-#define MYMPIISEND(a,b,c,d,e,f,g) MPI_Isend(a,b,c,d,e,f,g)
+//#define MYMPIISEND(a,b,c,d,e,f,g) MPI_Isend(a,b,c,d,e,f,g)
+#define MYMPIISEND(a,b,c,d,e,f,g) MPI_Send(a,b,c,d,e,f)
 #define MYMPIRECV(a,b,c,d,e,f,g) MPI_Recv(a,b,c,d,e,f,g)
 #define MYMPIBCAST(a,b,c,d,e) MPI_Bcast(a,b,c,d,e)
 #define MYMPIBARRIER(a) MPI_Barrier(a)
-#define MYMPIWAITALL(a,b,c) MPI_Waitall(a,b,c)
+//#define MYMPIWAITALL(a,b,c) MPI_Waitall(a,b,c)
+#define MYMPIWAITALL(a,b,c) /*do nothing*/
 #endif
 #endif
 #endif
