@@ -14,7 +14,7 @@
 Copyright 1996-2002 Peter Beerli and Joseph Felsenstein, Seattle WA
 Copyright 2003-2006 Peter Beerli, Tallahassee FL
  
-$Id: tools.h 1712 2010-08-20 04:17:16Z beerli $
+$Id: tools.h 1745 2010-10-24 20:56:28Z beerli $
 -------------------------------------------------------*/
 
 #include "migration.h"
@@ -89,6 +89,9 @@ extern void init_files (world_fmt * world, data_fmt * data,
 extern void exit_files (world_fmt * world, data_fmt * data,
                             option_fmt * options);
 extern void openfile (FILE ** fp, char *filename, char *mode, char *perm);
+#ifdef ZNZ
+extern void znzopenfile (znzFile * fp, char *filename, char *mode, int use_compressed);
+#endif
 extern long read_savesum (world_fmt * world, option_fmt * options,
                               data_fmt * data);
 extern void write_savesum (world_fmt * world);
@@ -102,6 +105,8 @@ extern void read_word(FILE *infile, char *word);
 extern void unread_word(FILE *infile, char *word);
 extern void add_to_buffer(char *fp, long *bufsize, char **buffer, long *allocbufsize);
 extern long print_to_buffer(char **buffer, long *maxbufsize, char *tempbuffer, long *pos, const char *fmt, ...);
+extern void record_warnings(world_fmt * world, const char *fmt, ...);
+extern void print_stored_warnings(world_fmt *world);
 extern void remove_trailing_blanks(char **filename);
 
 /* time reporting */
